Module: Kward::CLI::InteractiveTurn
- Included in:
- Kward::CLI
- Defined in:
- lib/kward/cli/interactive_turn.rb
Overview
Interactive turn loop helpers for streaming, cancellation, and queued user input.
Interactive turn loop helpers for streaming, cancellation, and queued user input.