Skip to content
Kward Search API index

Class: Kward::PluginRegistry::TransportType

Inherits:
Struct
  • Object
show all
Defined in:
lib/kward/plugin_registry.rb

Overview

Registered external transport. The factory receives a transport host and configuration when the transport is started, not while plugins load.

Instance Attribute Summary collapse

Instance Attribute Details

#capabilitiesObject

Returns the value of attribute capabilities

Returns:

  • (Object)

    the current value of capabilities



44
45
46
# File 'lib/kward/plugin_registry.rb', line 44

def capabilities
  @capabilities
end

#execution_profileObject

Returns the value of attribute execution_profile

Returns:

  • (Object)

    the current value of execution_profile



44
45
46
# File 'lib/kward/plugin_registry.rb', line 44

def execution_profile
  @execution_profile
end

#handlerObject

Returns the value of attribute handler

Returns:

  • (Object)

    the current value of handler



44
45
46
# File 'lib/kward/plugin_registry.rb', line 44

def handler
  @handler
end

#idObject

Returns the value of attribute id

Returns:

  • (Object)

    the current value of id



44
45
46
# File 'lib/kward/plugin_registry.rb', line 44

def id
  @id
end

#nameObject

Returns the value of attribute name

Returns:

  • (Object)

    the current value of name



44
45
46
# File 'lib/kward/plugin_registry.rb', line 44

def name
  @name
end

#pathObject

Returns the value of attribute path

Returns:

  • (Object)

    the current value of path



44
45
46
# File 'lib/kward/plugin_registry.rb', line 44

def path
  @path
end