Module: Kward::PromptInterface::RuntimeState
- Included in:
- Kward::PromptInterface
- Defined in:
- lib/kward/prompt_interface/runtime_state.rb
Overview
Runtime footer, model, persona, and session state shown by the prompt interface.
Runtime footer, model, persona, and session state shown by the prompt interface.