Skip to content
Kward Search API index

Class: Kward::PluginRegistry::InteractiveCommand

Inherits:
Struct
  • Object
show all
Defined in:
lib/kward/plugin_registry.rb

Overview

Registered interactive command that takes over the composer region with a Kward-driven render and input loop. Like a slash command but with canvas rendering capabilities for games, dashboards, viewers, and similar uses.

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#argument_hintObject

Returns the value of attribute argument_hint

Returns:

  • (Object)

    the current value of argument_hint



29
30
31
# File 'lib/kward/plugin_registry.rb', line 29

def argument_hint
  @argument_hint
end

#descriptionObject

Returns the value of attribute description

Returns:

  • (Object)

    the current value of description



29
30
31
# File 'lib/kward/plugin_registry.rb', line 29

def description
  @description
end

#fpsObject

Returns the value of attribute fps

Returns:

  • (Object)

    the current value of fps



29
30
31
# File 'lib/kward/plugin_registry.rb', line 29

def fps
  @fps
end

#handlerObject

Returns the value of attribute handler

Returns:

  • (Object)

    the current value of handler



29
30
31
# File 'lib/kward/plugin_registry.rb', line 29

def handler
  @handler
end

#nameObject

Returns the value of attribute name

Returns:

  • (Object)

    the current value of name



29
30
31
# File 'lib/kward/plugin_registry.rb', line 29

def name
  @name
end

#pathObject

Returns the value of attribute path

Returns:

  • (Object)

    the current value of path



29
30
31
# File 'lib/kward/plugin_registry.rb', line 29

def path
  @path
end

#rowsObject

Returns the value of attribute rows

Returns:

  • (Object)

    the current value of rows



29
30
31
# File 'lib/kward/plugin_registry.rb', line 29

def rows
  @rows
end

Instance Method Details

#entryObject



30
31
32
# File 'lib/kward/plugin_registry.rb', line 30

def entry
  { name: name, description: description, argument_hint: argument_hint }
end