Class: Kward::PluginRegistry::InteractiveCommand
- Inherits:
-
Struct
- Object
- Struct
- Kward::PluginRegistry::InteractiveCommand
- Defined in:
- lib/kward/plugin_registry.rb
Overview
Registered interactive command that takes over the composer region with a Kward-driven render and input loop. Like a slash command but with canvas rendering capabilities for games, dashboards, viewers, and similar uses.
Instance Attribute Summary collapse
-
#argument_hint ⇒ Object
Returns the value of attribute argument_hint.
-
#description ⇒ Object
Returns the value of attribute description.
-
#fps ⇒ Object
Returns the value of attribute fps.
-
#handler ⇒ Object
Returns the value of attribute handler.
-
#name ⇒ Object
Returns the value of attribute name.
-
#path ⇒ Object
Returns the value of attribute path.
-
#rows ⇒ Object
Returns the value of attribute rows.
Instance Method Summary collapse
Instance Attribute Details
#argument_hint ⇒ Object
Returns the value of attribute argument_hint
29 30 31 |
# File 'lib/kward/plugin_registry.rb', line 29 def argument_hint @argument_hint end |
#description ⇒ Object
Returns the value of attribute description
29 30 31 |
# File 'lib/kward/plugin_registry.rb', line 29 def description @description end |
#fps ⇒ Object
Returns the value of attribute fps
29 30 31 |
# File 'lib/kward/plugin_registry.rb', line 29 def fps @fps end |
#handler ⇒ Object
Returns the value of attribute handler
29 30 31 |
# File 'lib/kward/plugin_registry.rb', line 29 def handler @handler end |
#name ⇒ Object
Returns the value of attribute name
29 30 31 |
# File 'lib/kward/plugin_registry.rb', line 29 def name @name end |
#path ⇒ Object
Returns the value of attribute path
29 30 31 |
# File 'lib/kward/plugin_registry.rb', line 29 def path @path end |
#rows ⇒ Object
Returns the value of attribute rows
29 30 31 |
# File 'lib/kward/plugin_registry.rb', line 29 def rows @rows end |
Instance Method Details
#entry ⇒ Object
30 31 32 |
# File 'lib/kward/plugin_registry.rb', line 30 def entry { name: name, description: description, argument_hint: argument_hint } end |