Class: Kward::PluginRegistry::TransportType
- Inherits:
-
Struct
- Object
- Struct
- Kward::PluginRegistry::TransportType
- Defined in:
- lib/kward/plugin_registry.rb
Overview
Registered external transport. The factory receives a transport host and configuration when the transport is started, not while plugins load.
Instance Attribute Summary collapse
-
#capabilities ⇒ Object
Returns the value of attribute capabilities.
-
#execution_profile ⇒ Object
Returns the value of attribute execution_profile.
-
#handler ⇒ Object
Returns the value of attribute handler.
-
#id ⇒ Object
Returns the value of attribute id.
-
#name ⇒ Object
Returns the value of attribute name.
-
#path ⇒ Object
Returns the value of attribute path.
Instance Attribute Details
#capabilities ⇒ Object
Returns the value of attribute capabilities
44 45 46 |
# File 'lib/kward/plugin_registry.rb', line 44 def capabilities @capabilities end |
#execution_profile ⇒ Object
Returns the value of attribute execution_profile
44 45 46 |
# File 'lib/kward/plugin_registry.rb', line 44 def execution_profile @execution_profile end |
#handler ⇒ Object
Returns the value of attribute handler
44 45 46 |
# File 'lib/kward/plugin_registry.rb', line 44 def handler @handler end |
#id ⇒ Object
Returns the value of attribute id
44 45 46 |
# File 'lib/kward/plugin_registry.rb', line 44 def id @id end |
#name ⇒ Object
Returns the value of attribute name
44 45 46 |
# File 'lib/kward/plugin_registry.rb', line 44 def name @name end |
#path ⇒ Object
Returns the value of attribute path
44 45 46 |
# File 'lib/kward/plugin_registry.rb', line 44 def path @path end |