Class: Kward::PromptInterface
- Inherits:
-
Object
- Object
- Kward::PromptInterface
show all
- Includes:
- ApprovalPrompt, ComposerController, ComposerRenderer, EditorAutoClosePairs, EditorAutoIndent, EditorController, EditorEndwise, EditorRenderer, EditorRunner, EditorSyntaxHighlighter, EditorWordCompletion, EmacsEditorMode, FileOverlay, GitPrompt, InteractiveRenderer, InteractiveState, KeyHandler, Layout, ModernEditorMode, OverlayRenderer, ProjectBrowser, PromptRenderer, QuestionPrompt, RuntimeState, Screen, SelectionPrompt, SlashOverlay, TranscriptRenderer, VibeEditorMode, VibeInsertReadline
- Defined in:
- lib/kward/prompt_interface.rb,
lib/kward/prompt_interface/banner.rb,
lib/kward/prompt_interface/layout.rb,
lib/kward/prompt_interface/screen.rb,
lib/kward/prompt_interface/git_prompt.rb,
lib/kward/prompt_interface/key_handler.rb,
lib/kward/prompt_interface/editor/state.rb,
lib/kward/prompt_interface/file_overlay.rb,
lib/kward/prompt_interface/stream_state.rb,
lib/kward/prompt_interface/editor/buffer.rb,
lib/kward/prompt_interface/editor/runner.rb,
lib/kward/prompt_interface/editor/search.rb,
lib/kward/prompt_interface/runtime_state.rb,
lib/kward/prompt_interface/slash_overlay.rb,
lib/kward/prompt_interface/composer_state.rb,
lib/kward/prompt_interface/editor/endwise.rb,
lib/kward/prompt_interface/approval_prompt.rb,
lib/kward/prompt_interface/editor/renderer.rb,
lib/kward/prompt_interface/project_browser.rb,
lib/kward/prompt_interface/prompt_renderer.rb,
lib/kward/prompt_interface/question_prompt.rb,
lib/kward/prompt_interface/editor/kill_ring.rb,
lib/kward/prompt_interface/overlay_renderer.rb,
lib/kward/prompt_interface/selection_prompt.rb,
lib/kward/prompt_interface/composer_renderer.rb,
lib/kward/prompt_interface/editor/controller.rb,
lib/kward/prompt_interface/editor/modes/vibe.rb,
lib/kward/prompt_interface/editor/selections.rb,
lib/kward/prompt_interface/editor/vibe_state.rb,
lib/kward/prompt_interface/interactive/state.rb,
lib/kward/prompt_interface/transcript_buffer.rb,
lib/kward/prompt_interface/editor/auto_indent.rb,
lib/kward/prompt_interface/editor/file_marker.rb,
lib/kward/prompt_interface/editor/modes/emacs.rb,
lib/kward/prompt_interface/editor/status_text.rb,
lib/kward/prompt_interface/composer_controller.rb,
lib/kward/prompt_interface/editor/modes/modern.rb,
lib/kward/prompt_interface/editor/runner_state.rb,
lib/kward/prompt_interface/editor/undo_history.rb,
lib/kward/prompt_interface/transcript_renderer.rb,
lib/kward/prompt_interface/interactive/renderer.rb,
lib/kward/prompt_interface/editor/word_completion.rb,
lib/kward/prompt_interface/interactive/controller.rb,
lib/kward/prompt_interface/editor/auto_close_pairs.rb,
lib/kward/prompt_interface/editor/indent_navigation.rb,
lib/kward/prompt_interface/editor/syntax_highlighter.rb,
lib/kward/prompt_interface/editor/modes/vibe_insert_readline.rb
Overview
Interactive terminal UI used by the CLI frontend.
Defined Under Namespace
Modules: ApprovalPrompt, ComposerController, ComposerRenderer, EditorAutoClosePairs, EditorAutoIndent, EditorController, EditorEndwise, EditorRenderer, EditorRunner, EditorStatusText, EditorSyntaxHighlighter, EditorWordCompletion, EmacsEditorMode, FileOverlay, GitPrompt, InteractiveRenderer, InteractiveState, KeyHandler, Layout, ModernEditorMode, OverlayRenderer, ProjectBrowser, PromptRenderer, QuestionPrompt, RuntimeState, Screen, SelectionPrompt, SlashOverlay, TranscriptRenderer, VibeEditorMode, VibeInsertReadline
Classes: Banner, ComposerState, EditorBuffer, EditorFileMarker, EditorIndentNavigation, EditorKillRing, EditorRunnerState, EditorSearch, EditorSelections, EditorState, EditorUndoHistory, InteractiveController, StreamState, SubmittedInput, TranscriptBuffer, VibeEditorState
Constant Summary
collapse
- HELP_TEXT =
"Enter sends • Shift+Enter newline • / commands • @ files • ↑/↓ history • Ctrl+D exits".freeze
- BUSY_HELP_TEXT =
"Ctrl+C cancels".freeze
- SPINNER_FRAMES =
%w[· • ● •].freeze
- SPINNER_INTERVAL =
0.16
1.0
- COMPOSER_STATUS_REFRESH_INTERVAL =
1.0
- COMPLETION_DISPLAY_SECONDS =
0.8
- RESPONSE_ARRIVAL_SECONDS =
0.4
- BUSY_ELAPSED_DELAY_SECONDS =
2.0
- COMPOSER_MAX_INPUT_ROWS =
6
- IMAGE_VIEWER_MAX_ROWS =
8
- TRANSCRIPT_BUFFER_LIMIT =
200_000
- BANNER_MESSAGE =
Banner::MESSAGE
- KEYBOARD_PROTOCOL_ENABLE =
TerminalSequences::KEYBOARD_PROTOCOL_ENABLE
- KEYBOARD_PROTOCOL_RESTORE =
TerminalSequences::KEYBOARD_PROTOCOL_RESTORE
- BRACKETED_PASTE_ENABLE =
TerminalSequences::BRACKETED_PASTE_ENABLE
- BRACKETED_PASTE_RESTORE =
TerminalSequences::BRACKETED_PASTE_RESTORE
- BRACKETED_PASTE_START =
TerminalSequences::BRACKETED_PASTE_START
- BRACKETED_PASTE_END =
TerminalSequences::BRACKETED_PASTE_END
- SYNCHRONIZED_OUTPUT_ENABLE =
TerminalSequences::SYNCHRONIZED_OUTPUT_ENABLE
- SYNCHRONIZED_OUTPUT_DISABLE =
TerminalSequences::SYNCHRONIZED_OUTPUT_DISABLE
- CURSOR_SHOW =
TerminalSequences::CURSOR_SHOW
- CURSOR_HIDE =
TerminalSequences::CURSOR_HIDE
- CURSOR_SHAPE_DEFAULT =
TerminalSequences::CURSOR_SHAPE_DEFAULT
- CURSOR_SHAPE_BAR =
TerminalSequences::CURSOR_SHAPE_BAR
- SHIFT_ENTER_SEQUENCES =
TerminalKeys::SHIFT_ENTER
- EXIT_INPUT =
:exit_input
- CANCEL_INPUT =
:cancel_input
- SELECT_CANCEL =
:select_cancel
- SELECT_CONTINUE =
:select_continue
- SELECT_ACTION_MINIMUM_BUSY_SECONDS =
1.0
VibeEditorMode::VIBE_PAIR_TEXT_OBJECTS, VibeEditorMode::VIBE_RUBY_BLOCK_OPENERS, VibeEditorMode::VIBE_RUBY_EXTENSIONS, VibeEditorMode::VIBE_RUBY_PATHS, VibeEditorMode::VIBE_SIMPLE_MOTION_KEYS, VibeEditorMode::VIBE_VISUAL_LINE_MOVE_CONTROLS
EditorAutoIndent::C_LIKE_INDENT_LANGUAGES, EditorAutoIndent::EDITOR_SHIFT_TAB_SEQUENCES, EditorAutoIndent::EDITOR_TAB_SEQUENCES, EditorAutoIndent::LUA_INDENT_KEYWORDS, EditorAutoIndent::PUNCTUATION_INDENT_LANGUAGES, EditorAutoIndent::PUNCTUATION_PAIRS, EditorAutoIndent::PYTHON_INDENT_KEYWORDS, EditorAutoIndent::RUBY_INDENT_KEYWORDS, EditorAutoIndent::SHELL_DEDENT_KEYWORDS, EditorAutoIndent::SHELL_INDENT_KEYWORDS
EditorWordCompletion::EDITOR_COMPLETION_PREFIX_PATTERN, EditorWordCompletion::EDITOR_COMPLETION_SUFFIX_PATTERN, EditorWordCompletion::EDITOR_COMPLETION_WORD_PATTERN
EditorEndwise::ENDWISE_ENDLESS_DEFINITION, EditorEndwise::ENDWISE_LANGUAGES, EditorEndwise::ENDWISE_LINE_PARSE_LIMIT, EditorEndwise::ENDWISE_SINGLE_LINE_DEFINITION
EditorAutoClosePairs::AUTO_CLOSE_CLOSERS, EditorAutoClosePairs::AUTO_CLOSE_OPENERS, EditorAutoClosePairs::AUTO_CLOSE_PAIRS, EditorAutoClosePairs::AUTO_CLOSE_QUOTES, EditorAutoClosePairs::WORD_CHARACTER
EditorSyntaxHighlighter::CSS_EXTENSIONS, EditorSyntaxHighlighter::CSS_PATTERN, EditorSyntaxHighlighter::ERB_CLOSE_PATTERN, EditorSyntaxHighlighter::ERB_EXTENSIONS, EditorSyntaxHighlighter::ERB_OPEN_PATTERN, EditorSyntaxHighlighter::GENERIC_STRING_NUMBER_PATTERN, EditorSyntaxHighlighter::HTML_EXTENSIONS, EditorSyntaxHighlighter::HTML_TAG_START_PATTERN, EditorSyntaxHighlighter::JSON_EXTENSIONS, EditorSyntaxHighlighter::JSON_PATTERN, EditorSyntaxHighlighter::LANGUAGE_DEFINITIONS, EditorSyntaxHighlighter::MARKDOWN_EXTENSIONS, EditorSyntaxHighlighter::MARKDOWN_PATTERN, EditorSyntaxHighlighter::RUBY_EXTENSIONS, EditorSyntaxHighlighter::RUBY_FILENAMES, EditorSyntaxHighlighter::RUBY_KEYWORDS, EditorSyntaxHighlighter::RUBY_PATTERN, EditorSyntaxHighlighter::SCSS_EXTENSIONS, EditorSyntaxHighlighter::SQL_EXTENSIONS, EditorSyntaxHighlighter::SQL_PATTERN, EditorSyntaxHighlighter::YAML_EXTENSIONS, EditorSyntaxHighlighter::YAML_PATTERN
ProjectBrowser::IMAGE_VIEWER_CELL_HEIGHT_TO_WIDTH, ProjectBrowser::IMAGE_VIEWER_MAX_ZOOM, ProjectBrowser::IMAGE_VIEWER_MIN_ZOOM, ProjectBrowser::IMAGE_VIEWER_ZOOM_STEP, ProjectBrowser::PROJECT_BROWSER_DIRECTORY_ICON, ProjectBrowser::PROJECT_BROWSER_EXTENSION_ICONS, ProjectBrowser::PROJECT_BROWSER_FILENAME_ICONS, ProjectBrowser::PROJECT_BROWSER_FILE_ICON, ProjectBrowser::PROJECT_BROWSER_RESULT_LIMIT, ProjectBrowser::PROJECT_BROWSER_ROOT, ProjectBrowser::PROJECT_BROWSER_STATE_VERSION
Instance Method Summary
collapse
-
#ask(message = "You>") ⇒ Object
-
#ask_user_question(questions) ⇒ Object
-
#begin_busy_input(message = "You>", activity: "thinking") ⇒ Object
-
#clear_steered_count ⇒ Object
-
#clear_transcript ⇒ Object
-
#close ⇒ Object
-
#commit_editor_prompt(session) ⇒ Object
Commits an agent draft as one undoable editor operation.
-
#composer_snapshot ⇒ Object
-
#edit_file(path, base_dir: nil, allow_new: true) ⇒ Object
-
#editing_file? ⇒ Boolean
-
#editor_prompt_context ⇒ Object
Returns the active Vibe or Modern editor buffer for an agent prompt turn.
-
#filter_terminal_handoff_input(chunk) ⇒ Object
-
#finish_busy_input ⇒ Object
-
#finish_interactive ⇒ Object
-
#finish_stream_block ⇒ Object
-
#initialize(input: $stdin, output: $stdout, slash_commands: [], overlay_settings: nil, project_browser_icon_theme: "off", footer: nil, composer_status: nil, busy_help: true, attachment_badges: nil, attachment_parser: nil, banner_message: nil, tab_keybindings: nil, prompt_history: nil, workspace_root: Dir.pwd, editor_mode: nil, editor_mode_source: nil, editor_auto_indent: true, editor_auto_indent_source: nil, editor_auto_close_pairs: true, editor_auto_close_pairs_source: nil, editor_soft_wrap: true, editor_soft_wrap_source: nil, editor_bar_cursor: true, editor_bar_cursor_source: nil, editor_line_numbers: "absolute", editor_line_numbers_source: nil, diff_view: "auto", diff_view_source: nil, editor_runners_source: nil, redraw_handler: nil) ⇒ PromptInterface
constructor
A new instance of PromptInterface.
-
#inline_image_protocol ⇒ Object
-
#interactive_active? ⇒ Boolean
-
#interactive_exited? ⇒ Boolean
-
#modal_active? ⇒ Boolean
-
#new_composer_state_with_history ⇒ Object
-
#picker_choice_width ⇒ Object
-
#poll_input ⇒ Object
-
#prefill_input(value) ⇒ Object
Prefills the next composer input without submitting it.
-
#print_visual_banner(message = nil) ⇒ Object
-
#record_transient_terminal_output(text, render: true) ⇒ Object
-
#redraw ⇒ Object
-
#refresh_composer_status ⇒ Object
-
#restore_composer_snapshot(snapshot) ⇒ Object
-
#restore_composer_snapshot_locked(snapshot) ⇒ Object
-
#restore_editor_snapshot_locked(snapshot) ⇒ Object
-
#restore_tab_view_snapshot(snapshot) ⇒ Object
-
#restore_transcript ⇒ Object
-
#resume_editor_for_agent(status: nil) ⇒ Object
Resumes the editor after an agent prompt turn.
-
#review_document(title:, content:, &on_save) ⇒ Object
Opens an in-memory document editor.
-
#run_editor ⇒ Object
-
#say(message) ⇒ Object
-
#say_visual(message) ⇒ Object
-
#scratchpad(language = :text) ⇒ Object
-
#select(message, choices, title: "Sessions", custom: false, initial_index: 0, action_keys: {}, action_handlers: {}) ⇒ Object
-
#set_queued_count(count) ⇒ Object
-
#set_steered_count(count) ⇒ Object
-
#show_completion(status) ⇒ Object
-
#show_response_arrival ⇒ Object
-
#start(render: true) ⇒ Object
-
#start_interactive(title:, rows:, fps:) ⇒ Object
-
#start_stream_block(label) ⇒ Object
-
#suspend_editor_for_agent ⇒ Object
Suspends the live editor while an agent prompt is running.
-
#tab_view_snapshot(include_transcript: true) ⇒ Object
-
#update_assistant_label(label) ⇒ Object
-
#update_busy_activity(activity) ⇒ Object
-
#update_completion_provider(provider, slash_overlay: true) ⇒ Object
-
#update_overlay_settings(settings) ⇒ Object
-
#update_project_browser_icon_theme(theme) ⇒ Object
-
#update_prompt_label(label) ⇒ Object
Updates the visible prompt label while the composer is active.
-
#update_slash_commands(commands) ⇒ Object
-
#update_tabs(labels:, active_index: 0) ⇒ Object
-
#update_workspace_root(root, prompt_history: nil) ⇒ Object
-
#with_completion_provider(provider, slash_overlay: true) ⇒ Object
-
#with_inline_terminal_handoff(preserve_tab_keybindings: false, &block) ⇒ Object
-
#with_prompt_history(history) ⇒ Object
-
#with_terminal_handoff(inline: false, preserve_tab_keybindings: false) ⇒ Object
-
#write_delta(delta) ⇒ Object
-
#write_stream_block(label, delta, finish: false) ⇒ Object
-
#write_transcript(message) ⇒ Object
-
#write_transcript_delta(delta) ⇒ Object
-
#yes?(message, default: false) ⇒ Boolean
Methods included from GitPrompt
#git_commit_message, #open_modal_diff_viewer
#ask_tool_approval
#open_project_browser, #open_project_browser_locked
Constructor Details
#initialize(input: $stdin, output: $stdout, slash_commands: [], overlay_settings: nil, project_browser_icon_theme: "off", footer: nil, composer_status: nil, busy_help: true, attachment_badges: nil, attachment_parser: nil, banner_message: nil, tab_keybindings: nil, prompt_history: nil, workspace_root: Dir.pwd, editor_mode: nil, editor_mode_source: nil, editor_auto_indent: true, editor_auto_indent_source: nil, editor_auto_close_pairs: true, editor_auto_close_pairs_source: nil, editor_soft_wrap: true, editor_soft_wrap_source: nil, editor_bar_cursor: true, editor_bar_cursor_source: nil, editor_line_numbers: "absolute", editor_line_numbers_source: nil, diff_view: "auto", diff_view_source: nil, editor_runners_source: nil, redraw_handler: nil) ⇒ PromptInterface
Returns a new instance of PromptInterface.
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
|
# File 'lib/kward/prompt_interface.rb', line 143
def initialize(input: $stdin, output: $stdout, slash_commands: [], overlay_settings: nil, project_browser_icon_theme: "off", footer: nil, composer_status: nil, busy_help: true, attachment_badges: nil, attachment_parser: nil, banner_message: nil, tab_keybindings: nil, prompt_history: nil, workspace_root: Dir.pwd, editor_mode: nil, editor_mode_source: nil, editor_auto_indent: true, editor_auto_indent_source: nil, editor_auto_close_pairs: true, editor_auto_close_pairs_source: nil, editor_soft_wrap: true, editor_soft_wrap_source: nil, editor_bar_cursor: true, editor_bar_cursor_source: nil, editor_line_numbers: "absolute", editor_line_numbers_source: nil, diff_view: "auto", diff_view_source: nil, editor_runners_source: nil, redraw_handler: nil)
@input_io = input
@output_io = output
@terminal_owner = :kward
@terminal_handoff_mode = nil
@reader = TTY::Reader.new(input: input, output: output, interrupt: :error)
@mutex = Mutex.new
@prompt_history = prompt_history
@workspace_root = File.expand_path(workspace_root.to_s.empty? ? Dir.pwd : workspace_root)
@composer = ComposerState.new
load_history(@prompt_history.values) if @prompt_history
@started = false
@asking = false
@busy = false
@busy_activity = "thinking"
@busy_started_at = nil
@last_busy_elapsed = nil
@completion_status = nil
@response_arrival_until = nil
@queued_count = 0
@steered_count = 0
@spinner_frame_index = 0
@last_spinner_tick = monotonic_now
@last_footer_refresh = nil
@cached_footer_text = nil
@last_composer_status_refresh = 0.0
@cached_composer_status_text = nil
@prompt_label = "You>"
@assistant_label = "Assistant"
@stream_state = StreamState.new
@rendered_rows = 0
@last_composer_rows = []
@cursor_rendered_row = 0
@transcript_buffer = TranscriptBuffer.new(limit: TRANSCRIPT_BUFFER_LIMIT)
@transcript_viewport_rows = 0
@restoring_transcript = false
@pending_keys = []
@handoff_input_buffer = +"".b
@completion_provider = nil
@completion_cycle = nil
@completion_overlay = nil
@redraw_handler = redraw_handler
@transcript_redraw_requested = false
@original_console_mode = nil
@raw_mode_active = false
@slash_commands = normalize_slash_commands(slash_commands)
@slash_selection_index = 0
@slash_overlay_dismissed_input = nil
@slash_overlay_disabled = false
@file_selection_index = 0
@file_overlay_dismissed_token = nil
@file_open_dismissed_token = nil
@file_editor_open_status = nil
@file_mention_paths = nil
@file_mention_discovery_thread = nil
@file_mention_discovery_root = nil
@file_mention_discovery_token = nil
@project_browser_file_paths = nil
@project_browser_state = nil
@image_viewer_state = nil
@terminal_image_protocol = nil
@project_browser_restore_after_editor = false
@editor_state = nil
@editor_runner_state = nil
@editor_runner_completion = nil
@editor_runner_output_visible = false
@editor_runner_selection_anchor = nil
@editor_runner_selection_cursor = nil
@editor_runner_mouse_dragging = false
@editor_runner_last_click = nil
@editor_agent_suspended = false
@editor_prompt_active = false
@editor_prompt_input = ""
@interactive_state = nil
@last_interactive_tick = monotonic_now
@select_state = nil
@question_state = nil
@question_prompt_active = false
@git_state = nil
@last_width = screen_width
@last_height = screen_height
@reserved_rows = 0
@color_enabled = ANSI.enabled?(output)
@cursor_visible = true
@editor_bar_cursor_active = false
@synchronized_output_depth = 0
@overlay_settings = normalize_overlay_settings(overlay_settings)
@project_browser_icon_theme = normalize_project_browser_icon_theme(project_browser_icon_theme)
@footer =
@composer_status = composer_status
@busy_help = busy_help
@attachment_badges = attachment_badges
@attachment_parser = attachment_parser
@banner = Banner.new(message: banner_message, screen_height: method(:screen_height))
@tabs = []
@active_tab_index = 0
@tab_keybindings = normalize_tab_keybindings(tab_keybindings)
@editor_mode = normalize_editor_mode(editor_mode)
@editor_mode_source = editor_mode_source
@editor_auto_indent = editor_auto_indent != false
@editor_auto_indent_source = editor_auto_indent_source
@editor_auto_close_pairs = editor_auto_close_pairs != false
@editor_auto_close_pairs_source = editor_auto_close_pairs_source
@editor_soft_wrap = editor_soft_wrap != false
@editor_soft_wrap_source = editor_soft_wrap_source
@editor_bar_cursor = editor_bar_cursor != false
@editor_bar_cursor_source = editor_bar_cursor_source
@editor_line_numbers = normalize_editor_line_numbers(editor_line_numbers)
@editor_line_numbers_source = editor_line_numbers_source
@diff_view = DiffViewMode.normalize(diff_view)
@diff_view_source = diff_view_source
@editor_runners_source = editor_runners_source
end
|
Instance Method Details
#ask(message = "You>") ⇒ Object
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
|
# File 'lib/kward/prompt_interface.rb', line 525
def ask(message = "You>")
was_composing = @started && @asking
start
@mutex.synchronize do
preserve_input = was_composing && !@busy && !composer_input.empty?
@prompt_label = message.to_s
unless preserve_input
self.composer_input = @composer.prefill_input.to_s
@composer.prefill_input = nil
self.composer_cursor = composer_input.length
@composer.clear_attachments
reset_history_navigation
end
@pending_keys.clear
@completion_cycle = nil
@completion_overlay = nil
@asking = true
@busy = false
@queued_count = 0
render_prompt_locked
end
loop do
key = read_key(nonblock: true)
result = nil
@mutex.synchronize do
if key.nil?
resized = handle_resize_locked
=
completion_changed = tick_completion_locked
render_prompt_locked if resized || || completion_changed
else
result = handle_key(key)
render_prompt_locked unless result.is_a?(String) || result == EXIT_INPUT || prompt_action_result?(result)
end
end
perform_pending_transcript_redraw
return result if result.is_a?(String) || prompt_action_result?(result)
return nil if result == EXIT_INPUT
sleep 0.02 if key.nil?
end
ensure
perform_pending_transcript_redraw
end
|
#ask_user_question(questions) ⇒ Object
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
|
# File 'lib/kward/prompt_interface.rb', line 643
def ask_user_question(questions)
return [] if questions.empty?
start
saved_state = nil
answers = []
@mutex.synchronize do
@question_prompt_active = true
saved_state = begin_question_prompt_state
end
questions.each_with_index do |question, index|
answer = ask_single_user_question(question, index + 1, questions.length)
if answer == SELECT_CANCEL
finish_question_prompt(saved_state)
return nil
end
answers << answer
end
finish_question_prompt(saved_state)
answers
rescue StandardError
finish_question_prompt(saved_state) if saved_state
raise
end
|
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
|
# File 'lib/kward/prompt_interface.rb', line 875
def begin_busy_input(message = "You>", activity: "thinking")
start
@mutex.synchronize do
@prompt_label = message.to_s
@busy_activity = normalize_busy_activity(activity)
@completion_status = nil
@response_arrival_until = nil
self.composer_input = ""
self.composer_cursor = 0
@composer.clear_attachments
@pending_keys.clear
@asking = true
@busy = true
@busy_started_at = monotonic_now
@last_busy_elapsed = nil
@queued_count = 0
@steered_count = 0
reset_spinner_locked
reset_history_navigation
render_prompt_locked
end
end
|
#clear_steered_count ⇒ Object
925
926
927
928
929
930
931
|
# File 'lib/kward/prompt_interface.rb', line 925
def clear_steered_count
@mutex.synchronize do
@steered_count = 0
@busy_activity = "thinking"
render_prompt_locked if @asking
end
end
|
#clear_transcript ⇒ Object
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
|
# File 'lib/kward/prompt_interface.rb', line 1100
def clear_transcript
@mutex.synchronize do
@transcript_buffer.clear
@transcript_viewport_rows = 0
@stream_state.finish_block
@stream_state.reset
width, height = screen_size
with_synchronized_output_locked { redraw_screen_locked(width: width, height: height) }
flush_output_locked
end
end
|
#close ⇒ Object
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
|
# File 'lib/kward/prompt_interface.rb', line 281
def close
@mutex.synchronize do
return unless @started
clear_image_viewer_output_locked if image_viewer_active?
clear_prompt_for_output_locked
restore_scroll_region_locked
disable_editor_mouse_reporting(force: true)
print_output_locked(BRACKETED_PASTE_RESTORE)
print_output_locked(KEYBOARD_PROTOCOL_RESTORE)
restore_editor_cursor_shape_locked
set_cursor_visible_locked(true, force: true)
puts_output_locked
flush_output_locked
@started = false
restore_console_mode_locked
end
end
|
#commit_editor_prompt(session) ⇒ Object
Commits an agent draft as one undoable editor operation.
412
413
414
415
416
417
418
419
420
421
422
|
# File 'lib/kward/prompt_interface.rb', line 412
def commit_editor_prompt(session)
@mutex.synchronize do
return false unless editor_active? && @editor_agent_suspended
return false unless @editor_state.buffer == session.initial_content
changed = @editor_state.replace_buffer(session.content)
@editor_state.status = changed ? "Agent updated buffer · :w save" : "Agent made no buffer changes"
render_prompt_locked if @started
changed
end
end
|
#composer_snapshot ⇒ Object
785
786
787
788
789
790
791
792
|
# File 'lib/kward/prompt_interface.rb', line 785
def composer_snapshot
@mutex.synchronize do
{
composer: @composer,
prompt_label: @prompt_label
}
end
end
|
#edit_file(path, base_dir: nil, allow_new: true) ⇒ Object
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
|
# File 'lib/kward/prompt_interface.rb', line 453
def edit_file(path, base_dir: nil, allow_new: true)
start(render: false)
opened = @mutex.synchronize do
open_editor(path, allow_new: allow_new, base_dir: base_dir || prompt_workspace_root, restrict_to_workspace: false).tap do
render_prompt_locked
end
end
return false unless opened
run_editor
rescue StandardError
@mutex.synchronize do
close_editor if editor_active?
render_prompt_locked if @started
end
raise
end
|
#editing_file? ⇒ Boolean
378
379
380
|
# File 'lib/kward/prompt_interface.rb', line 378
def editing_file?
@mutex.synchronize { editor_active? }
end
|
#editor_prompt_context ⇒ Object
Returns the active Vibe or Modern editor buffer for an agent prompt turn.
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
|
# File 'lib/kward/prompt_interface.rb', line 383
def editor_prompt_context
@mutex.synchronize do
next nil unless editor_active? && (@editor_state.vibe? || @editor_state.modern?)
{
path: @editor_state.path,
display_path: @editor_state.display_path,
workspace_root: @workspace_root,
language: @editor_state.language,
content: @editor_state.buffer.dup,
cursor: @editor_state.cursor_line_and_column,
selection: @editor_state.selected_text
}
end
end
|
961
962
963
|
# File 'lib/kward/prompt_interface.rb', line 961
def filter_terminal_handoff_input(chunk)
@mutex.synchronize { filter_terminal_handoff_input_locked(chunk) }
end
|
933
934
935
936
937
938
939
940
941
942
943
944
945
|
# File 'lib/kward/prompt_interface.rb', line 933
def finish_busy_input
@mutex.synchronize do
@last_busy_elapsed = monotonic_now - @busy_started_at if @busy_started_at
@busy_started_at = nil
@busy = false
@busy_activity = "thinking"
@response_arrival_until = nil
@queued_count = 0
@steered_count = 0
@asking = true
render_prompt_locked
end
end
|
#finish_interactive ⇒ Object
686
687
688
689
690
691
692
693
694
695
696
|
# File 'lib/kward/prompt_interface.rb', line 686
def finish_interactive
@mutex.synchronize do
return unless @interactive_state
snapshot = @interactive_state[:snapshot]
@interactive_state = nil
restore_composer_snapshot_locked(snapshot)
redraw_screen_locked if @started
flush_output_locked
end
end
|
#finish_stream_block ⇒ Object
1042
1043
1044
1045
1046
|
# File 'lib/kward/prompt_interface.rb', line 1042
def finish_stream_block
@mutex.synchronize do
write_stream_block_locked(nil, "", finish: true)
end
end
|
#inline_image_protocol ⇒ Object
436
437
438
|
# File 'lib/kward/prompt_interface.rb', line 436
def inline_image_protocol
@mutex.synchronize { terminal_image_protocol }
end
|
#interactive_active? ⇒ Boolean
674
675
676
|
# File 'lib/kward/prompt_interface.rb', line 674
def interactive_active?
@mutex.synchronize { interactive_active_locked? }
end
|
#interactive_exited? ⇒ Boolean
678
679
680
681
682
683
684
|
# File 'lib/kward/prompt_interface.rb', line 678
def interactive_exited?
@mutex.synchronize do
return false unless @interactive_state
@interactive_state[:controller].exited?
end
end
|
#modal_active? ⇒ Boolean
670
671
672
|
# File 'lib/kward/prompt_interface.rb', line 670
def modal_active?
@mutex.synchronize { modal_active_locked? }
end
|
#new_composer_state_with_history ⇒ Object
838
839
840
841
842
|
# File 'lib/kward/prompt_interface.rb', line 838
def new_composer_state_with_history
composer = ComposerState.new
composer.load_history(@prompt_history.values) if @prompt_history
composer
end
|
#picker_choice_width ⇒ Object
596
597
598
|
# File 'lib/kward/prompt_interface.rb', line 596
def picker_choice_width
[overlay_card_width(screen_width) - 6, 1].max
end
|
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
989
990
991
992
993
994
995
996
997
998
999
1000
1001
1002
1003
1004
|
# File 'lib/kward/prompt_interface.rb', line 965
def poll_input
key = read_key(nonblock: true)
@mutex.synchronize do
if interactive_active_locked?
if key.nil?
resized = handle_resize_locked
ticked = tick_interactive_locked
render_prompt_locked if resized || ticked
return :interactive_exited if @interactive_state[:controller].exited?
return nil
end
route_interactive_key(key)
ticked = tick_interactive_locked
render_prompt_locked if ticked
return :interactive_exited if @interactive_state[:controller].exited?
return nil
end
if key.nil?
resized = handle_resize_locked
spun = tick_spinner_locked
=
completion_changed = tick_completion_locked
render_prompt_locked if resized || spun || || completion_changed
return nil
end
if modal_active_locked?
queue_pending_keys(key)
return nil
end
result = handle_key(key)
render_prompt_locked unless [EXIT_INPUT, CANCEL_INPUT].include?(result) || prompt_action_result?(result)
[EXIT_INPUT, CANCEL_INPUT].include?(result) ? result : result
end
ensure
perform_pending_transcript_redraw
end
|
Prefills the next composer input without submitting it.
572
573
574
575
576
|
# File 'lib/kward/prompt_interface.rb', line 572
def prefill_input(value)
@mutex.synchronize do
@composer.prefill_input = value.to_s
end
end
|
#print_visual_banner(message = nil) ⇒ Object
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
|
# File 'lib/kward/prompt_interface.rb', line 1012
def print_visual_banner(message = nil)
@mutex.synchronize do
width, height = screen_size
rows = banner_rows(width, message: message)
return if rows.empty?
with_synchronized_output_locked do
prepare_transcript_output_locked
write_transcript_text_locked(rows.join("\n"))
write_transcript_text_locked("\n")
remember_transcript_viewport_locked(height)
@stream_state.finish_block
restore_composer_cursor_locked
end
flush_output_locked
end
end
|
#record_transient_terminal_output(text, render: true) ⇒ Object
1048
1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
1060
1061
1062
1063
|
# File 'lib/kward/prompt_interface.rb', line 1048
def record_transient_terminal_output(text, render: true)
value = text.to_s
return if value.empty?
@mutex.synchronize do
append_transcript_buffer(value)
append_transcript_buffer("\n") unless value.end_with?("\n")
@stream_state.finish_block
width, height = screen_size
remember_transcript_viewport_locked(height)
next unless render
with_synchronized_output_locked { redraw_screen_locked(width: width, height: height) }
flush_output_locked
end
end
|
#redraw ⇒ Object
1082
1083
1084
1085
1086
1087
1088
|
# File 'lib/kward/prompt_interface.rb', line 1082
def redraw
@mutex.synchronize do
width, height = screen_size
with_synchronized_output_locked { redraw_screen_locked(width: width, height: height) }
flush_output_locked
end
end
|
#refresh_composer_status ⇒ Object
1090
1091
1092
1093
1094
1095
1096
1097
1098
|
# File 'lib/kward/prompt_interface.rb', line 1090
def refresh_composer_status
@mutex.synchronize do
@cached_footer_text = nil
@last_footer_refresh = nil
@cached_composer_status_text = nil
@last_composer_status_refresh = 0.0
render_prompt_locked if @started && @asking
end
end
|
#restore_composer_snapshot(snapshot) ⇒ Object
810
811
812
813
814
815
816
|
# File 'lib/kward/prompt_interface.rb', line 810
def restore_composer_snapshot(snapshot)
@mutex.synchronize do
restore_composer_snapshot_locked(snapshot)
restore_editor_snapshot_locked(snapshot)
redraw_screen_locked if @started
end
end
|
#restore_composer_snapshot_locked(snapshot) ⇒ Object
830
831
832
833
834
835
836
|
# File 'lib/kward/prompt_interface.rb', line 830
def restore_composer_snapshot_locked(snapshot)
@composer = snapshot[:composer] || new_composer_state_with_history
@completion_cycle = nil
@completion_overlay = nil
@prompt_label = snapshot[:prompt_label].to_s.empty? ? "You>" : snapshot[:prompt_label].to_s
@last_composer_rows = []
end
|
#restore_editor_snapshot_locked(snapshot) ⇒ Object
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
859
|
# File 'lib/kward/prompt_interface.rb', line 844
def restore_editor_snapshot_locked(snapshot)
editor_was_active = editor_active?
cancel_editor_runner
@editor_runner_state = nil
@editor_runner_completion = nil
@editor_runner_output_visible = false
clear_editor_runner_selection
@editor_state = snapshot[:editor_state]&.dup
editor_is_active = editor_active?
if editor_is_active
enable_editor_mouse_reporting unless editor_was_active
else
disable_editor_mouse_reporting(force: true)
end
end
|
#restore_tab_view_snapshot(snapshot) ⇒ Object
818
819
820
821
822
823
824
825
826
827
828
|
# File 'lib/kward/prompt_interface.rb', line 818
def restore_tab_view_snapshot(snapshot)
@mutex.synchronize do
restore_composer_snapshot_locked(snapshot)
restore_editor_snapshot_locked(snapshot)
@transcript_buffer = snapshot[:transcript_buffer] || TranscriptBuffer.new(limit: TRANSCRIPT_BUFFER_LIMIT)
@transcript_viewport_rows = snapshot[:transcript_viewport_rows].to_i
@stream_state = snapshot[:stream_state] || StreamState.new
@last_composer_rows = []
redraw_screen_locked if @started
end
end
|
#restore_transcript ⇒ Object
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
|
# File 'lib/kward/prompt_interface.rb', line 324
def restore_transcript
start(render: false) unless @started
@mutex.synchronize do
print_output_locked(SYNCHRONIZED_OUTPUT_ENABLE)
clear_prompt_for_output_locked unless @rendered_rows.zero? && @last_composer_rows.empty?
@transcript_buffer.clear
@transcript_viewport_rows = 0
@stream_state.finish_block
@stream_state.reset
@restoring_transcript = true
end
yield
ensure
@mutex.synchronize do
@restoring_transcript = false
width, height = screen_size
redraw_screen_locked(width: width, height: height)
print_output_locked(SYNCHRONIZED_OUTPUT_DISABLE)
flush_output_locked
end
end
|
#resume_editor_for_agent(status: nil) ⇒ Object
Resumes the editor after an agent prompt turn.
425
426
427
428
429
430
431
432
433
434
|
# File 'lib/kward/prompt_interface.rb', line 425
def resume_editor_for_agent(status: nil)
@mutex.synchronize do
return false unless editor_active?
@editor_agent_suspended = false
@editor_state.status = status.to_s unless status.to_s.empty?
render_prompt_locked if @started
true
end
end
|
#review_document(title:, content:, &on_save) ⇒ Object
Opens an in-memory document editor. The callback receives edited content
when the user saves; return an error message to keep the editor open.
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
|
# File 'lib/kward/prompt_interface.rb', line 485
def review_document(title:, content:, &on_save)
raise ArgumentError, "review_document requires a save callback" unless on_save
start(render: false)
opened = @mutex.synchronize do
@editor_save_callback = on_save
open_scratchpad(:markdown, content: content).tap do
@editor_state.status = "#{title} · Ctrl+S save · Ctrl+Q cancel"
render_prompt_locked
end
end
return false unless opened
run_editor
end
|
#run_editor ⇒ Object
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
|
# File 'lib/kward/prompt_interface.rb', line 501
def run_editor
loop do
key = read_key(nonblock: true)
action = nil
editor_open = @mutex.synchronize do
if key.nil?
resized = handle_resize_locked
=
render_prompt_locked if resized ||
else
result = handle_key(key)
action = result if prompt_action_result?(result)
render_prompt_locked unless result.is_a?(String) || result == EXIT_INPUT || prompt_action_result?(result)
end
editor_active?
end
return action if action
break unless editor_open
sleep 0.02 if key.nil?
end
true
end
|
#say(message) ⇒ Object
300
301
302
|
# File 'lib/kward/prompt_interface.rb', line 300
def say(message)
write_transcript_message(message, preserve_composer: false)
end
|
#say_visual(message) ⇒ Object
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
|
# File 'lib/kward/prompt_interface.rb', line 308
def say_visual(message)
@mutex.synchronize do
return if @restoring_transcript
with_synchronized_output_locked do
clear_prompt_for_output_locked
text = message.to_s
write_visual_transcript_text_locked(text)
write_visual_transcript_text_locked("\n") unless text.end_with?("\n")
@stream_state.finish_block
render_prompt_after_output_locked
end
flush_output_locked
end
end
|
#scratchpad(language = :text) ⇒ Object
471
472
473
474
475
476
477
478
479
480
481
|
# File 'lib/kward/prompt_interface.rb', line 471
def scratchpad(language = :text)
start(render: false)
opened = @mutex.synchronize do
open_scratchpad(language).tap do
render_prompt_locked
end
end
return false unless opened
run_editor
end
|
#select(message, choices, title: "Sessions", custom: false, initial_index: 0, action_keys: {}, action_handlers: {}) ⇒ Object
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
|
# File 'lib/kward/prompt_interface.rb', line 600
def select(message, choices, title: "Sessions", custom: false, initial_index: 0, action_keys: {}, action_handlers: {})
return nil if choices.empty? && !custom
start
@mutex.synchronize do
prepare_modal_input_locked(message, clear_attachments: true)
choice_labels = choices.map(&:to_s)
selection_index = choice_labels.empty? ? 0 : [[initial_index.to_i, 0].max, choice_labels.length - 1].min
@select_state = { choices: choice_labels, selection_index: selection_index, title: title.to_s, custom: custom, action_keys: normalized_select_action_keys(action_keys), search_active: false }
render_prompt_locked
end
loop do
key = read_key(nonblock: true)
result = nil
@mutex.synchronize do
if key.nil?
resized = handle_resize_locked
=
render_prompt_locked if resized ||
else
result = handle_select_key(key)
result = drain_pending_select_keys_locked(result)
render_prompt_locked unless result.is_a?(String) || select_action_result?(result) || result == SELECT_CANCEL
end
end
if select_action_result?(result) && select_action_handler(result, action_handlers)
action_result = run_select_action(result, select_action_handler(result, action_handlers))
next if action_result == SELECT_CONTINUE
return action_result
end
if result.is_a?(String) || select_action_result?(result) || result == SELECT_CANCEL
finish_select_prompt(render: !select_deferred_finish_render?(result))
return result == SELECT_CANCEL ? nil : result
end
sleep 0.02 if key.nil?
end
end
|
#set_queued_count(count) ⇒ Object
909
910
911
912
913
914
915
|
# File 'lib/kward/prompt_interface.rb', line 909
def set_queued_count(count)
@mutex.synchronize do
@queued_count = count.to_i
@steered_count = 0 if @queued_count.positive?
render_prompt_locked if @asking
end
end
|
#set_steered_count(count) ⇒ Object
917
918
919
920
921
922
923
|
# File 'lib/kward/prompt_interface.rb', line 917
def set_steered_count(count)
@mutex.synchronize do
@steered_count = count.to_i
@queued_count = 0 if @steered_count.positive?
render_prompt_locked if @asking
end
end
|
#show_completion(status) ⇒ Object
954
955
956
957
958
959
|
# File 'lib/kward/prompt_interface.rb', line 954
def show_completion(status)
@mutex.synchronize do
@completion_status = completion_status(status)
render_prompt_locked if @started && @asking
end
end
|
#show_response_arrival ⇒ Object
947
948
949
950
951
952
|
# File 'lib/kward/prompt_interface.rb', line 947
def show_response_arrival
@mutex.synchronize do
@response_arrival_until = monotonic_now + RESPONSE_ARRIVAL_SECONDS
render_prompt_locked if @started && @asking && @busy
end
end
|
#start(render: true) ⇒ Object
267
268
269
270
271
272
273
274
275
276
277
278
279
|
# File 'lib/kward/prompt_interface.rb', line 267
def start(render: true)
@mutex.synchronize do
return if @started
enter_raw_mode_locked
@started = true
@asking = true
disable_editor_mouse_reporting(force: true) unless editor_active?
print_output_locked(KEYBOARD_PROTOCOL_ENABLE)
print_output_locked(BRACKETED_PASTE_ENABLE)
render_prompt_locked if render
end
end
|
#start_interactive(title:, rows:, fps:) ⇒ Object
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
|
# File 'lib/kward/prompt_interface.rb', line 752
def start_interactive(title:, rows:, fps:)
snapshot = composer_snapshot
controller = InteractiveController.new(width: interactive_canvas_width, height: rows, fps: fps)
start
@mutex.synchronize do
@interactive_state = {
title: title.to_s,
rows: rows,
controller: controller,
snapshot: snapshot
}
@last_interactive_tick = monotonic_now
@asking = true
@busy = false
@last_composer_rows = []
render_prompt_locked
end
controller
end
|
#start_stream_block(label) ⇒ Object
1030
1031
1032
1033
1034
|
# File 'lib/kward/prompt_interface.rb', line 1030
def start_stream_block(label)
@mutex.synchronize do
write_stream_block_locked(label, "", finish: false)
end
end
|
#suspend_editor_for_agent ⇒ Object
Suspends the live editor while an agent prompt is running.
400
401
402
403
404
405
406
407
408
409
|
# File 'lib/kward/prompt_interface.rb', line 400
def suspend_editor_for_agent
@mutex.synchronize do
return false unless editor_active? && !@editor_agent_suspended
@editor_agent_suspended = true
@editor_state.status = "Agent is working · Ctrl+C cancels"
render_prompt_locked if @started
true
end
end
|
#tab_view_snapshot(include_transcript: true) ⇒ Object
794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
|
# File 'lib/kward/prompt_interface.rb', line 794
def tab_view_snapshot(include_transcript: true)
@mutex.synchronize do
snapshot = {
composer: @composer.dup,
prompt_label: @prompt_label.dup,
editor_state: @editor_state&.dup
}
if include_transcript
snapshot[:transcript_buffer] = @transcript_buffer.dup
snapshot[:transcript_viewport_rows] = @transcript_viewport_rows
snapshot[:stream_state] = @stream_state.dup
end
snapshot
end
end
|
#update_assistant_label(label) ⇒ Object
1006
1007
1008
1009
1010
|
# File 'lib/kward/prompt_interface.rb', line 1006
def update_assistant_label(label)
@mutex.synchronize do
@assistant_label = label.to_s.empty? ? "Assistant" : label.to_s
end
end
|
#update_busy_activity(activity) ⇒ Object
898
899
900
901
902
903
904
905
906
907
|
# File 'lib/kward/prompt_interface.rb', line 898
def update_busy_activity(activity)
@mutex.synchronize do
value = normalize_busy_activity(activity)
return false if value == @busy_activity
@busy_activity = value
render_prompt_locked if @started && @asking && @busy
true
end
end
|
#update_completion_provider(provider, slash_overlay: true) ⇒ Object
358
359
360
361
362
363
364
365
366
|
# File 'lib/kward/prompt_interface.rb', line 358
def update_completion_provider(provider, slash_overlay: true)
@mutex.synchronize do
@completion_provider = provider
@completion_cycle = nil
@completion_overlay = nil
@slash_overlay_disabled = !slash_overlay
render_prompt_locked if @started
end
end
|
#update_overlay_settings(settings) ⇒ Object
861
862
863
864
865
866
|
# File 'lib/kward/prompt_interface.rb', line 861
def update_overlay_settings(settings)
@mutex.synchronize do
@overlay_settings = normalize_overlay_settings(settings)
render_prompt_locked if @started && @asking
end
end
|
#update_project_browser_icon_theme(theme) ⇒ Object
868
869
870
871
872
873
|
# File 'lib/kward/prompt_interface.rb', line 868
def update_project_browser_icon_theme(theme)
@mutex.synchronize do
@project_browser_icon_theme = normalize_project_browser_icon_theme(theme)
render_prompt_locked if @started && @asking && project_browser_visible?
end
end
|
#update_prompt_label(label) ⇒ Object
Updates the visible prompt label while the composer is active.
579
580
581
582
583
584
|
# File 'lib/kward/prompt_interface.rb', line 579
def update_prompt_label(label)
@mutex.synchronize do
@prompt_label = label.to_s
render_prompt_locked if @started && @asking
end
end
|
#update_slash_commands(commands) ⇒ Object
258
259
260
261
262
263
264
265
|
# File 'lib/kward/prompt_interface.rb', line 258
def update_slash_commands(commands)
@mutex.synchronize do
@slash_commands = normalize_slash_commands(commands)
reset_slash_selection
@slash_overlay_dismissed_input = nil
render_prompt_locked if @started
end
end
|
#update_tabs(labels:, active_index: 0) ⇒ Object
772
773
774
775
776
777
778
779
780
781
782
783
|
# File 'lib/kward/prompt_interface.rb', line 772
def update_tabs(labels:, active_index: 0)
@mutex.synchronize do
normalized_labels = Array(labels).map { |label| normalize_tab_label(label) }
normalized_index = active_index.to_i
return false if @tabs == normalized_labels && @active_tab_index == normalized_index
@tabs = normalized_labels
@active_tab_index = normalized_index
render_prompt_locked if @started && @asking
true
end
end
|
#update_workspace_root(root, prompt_history: nil) ⇒ Object
440
441
442
443
444
445
446
447
448
449
450
451
|
# File 'lib/kward/prompt_interface.rb', line 440
def update_workspace_root(root, prompt_history: nil)
@mutex.synchronize do
expanded_root = File.expand_path(root.to_s.empty? ? Dir.pwd : root)
return false if expanded_root == @workspace_root
@workspace_root = expanded_root
@prompt_history = prompt_history if prompt_history
load_history(@prompt_history.values) if @prompt_history
reset_workspace_file_state_locked
true
end
end
|
#with_completion_provider(provider, slash_overlay: true) ⇒ Object
347
348
349
350
351
352
353
354
355
356
|
# File 'lib/kward/prompt_interface.rb', line 347
def with_completion_provider(provider, slash_overlay: true)
previous_provider = @completion_provider
previous_slash_overlay_disabled = @slash_overlay_disabled
@completion_provider = provider
@slash_overlay_disabled = !slash_overlay
yield
ensure
@completion_provider = previous_provider
@slash_overlay_disabled = previous_slash_overlay_disabled
end
|
#with_inline_terminal_handoff(preserve_tab_keybindings: false, &block) ⇒ Object
698
699
700
|
# File 'lib/kward/prompt_interface.rb', line 698
def with_inline_terminal_handoff(preserve_tab_keybindings: false, &block)
with_terminal_handoff(inline: true, preserve_tab_keybindings: preserve_tab_keybindings, &block)
end
|
#with_prompt_history(history) ⇒ Object
368
369
370
371
372
373
374
375
376
|
# File 'lib/kward/prompt_interface.rb', line 368
def with_prompt_history(history)
previous_history = @prompt_history
@prompt_history = history
load_history(@prompt_history.values) if @prompt_history
yield
ensure
@prompt_history = previous_history
load_history(@prompt_history.values) if @prompt_history
end
|
#with_terminal_handoff(inline: false, preserve_tab_keybindings: false) ⇒ Object
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
|
# File 'lib/kward/prompt_interface.rb', line 702
def with_terminal_handoff(inline: false, preserve_tab_keybindings: false)
start
input = nil
output = nil
transition = nil
@mutex.synchronize do
if inline
handle_resize_locked
reserve_composer_region_locked
move_to_transcript_cursor_locked
else
clear_prompt_for_output_locked
restore_scroll_region_locked
end
disable_editor_mouse_reporting(force: true)
print_output_locked(BRACKETED_PASTE_RESTORE)
print_output_locked(KEYBOARD_PROTOCOL_RESTORE) unless preserve_tab_keybindings
@handoff_input_buffer.clear
restore_editor_cursor_shape_locked
set_cursor_visible_locked(true, force: true)
flush_output_locked
restore_console_mode_locked
input = @input_io
output = @output_io
transition = method(:switch_terminal_handoff_to_exclusive)
@terminal_handoff_mode = inline ? :inline : :exclusive
@terminal_owner = :child
end
yield(input, output, transition)
ensure
@mutex.synchronize do
@terminal_owner = :kward
@terminal_handoff_mode = nil
restore_scroll_region_locked
print_output_locked(TerminalSequences::SGR_RESET)
enter_raw_mode_locked
print_output_locked(KEYBOARD_PROTOCOL_ENABLE)
print_output_locked(BRACKETED_PASTE_ENABLE)
@handoff_input_buffer.clear
disable_editor_mouse_reporting(force: true)
restore_editor_cursor_shape_locked(force: true)
@cursor_visible = nil
@last_composer_rows = []
width, height = screen_size
with_synchronized_output_locked { redraw_screen_locked(width: width, height: height) }
flush_output_locked
end
end
|
#write_delta(delta) ⇒ Object
1036
1037
1038
1039
1040
|
# File 'lib/kward/prompt_interface.rb', line 1036
def write_delta(delta)
@mutex.synchronize do
write_stream_block_locked(nil, delta.to_s, finish: false)
end
end
|
#write_stream_block(label, delta, finish: false) ⇒ Object
1076
1077
1078
1079
1080
|
# File 'lib/kward/prompt_interface.rb', line 1076
def write_stream_block(label, delta, finish: false)
@mutex.synchronize do
write_stream_block_locked(label, delta.to_s, finish: finish)
end
end
|
#write_transcript(message) ⇒ Object
304
305
306
|
# File 'lib/kward/prompt_interface.rb', line 304
def write_transcript(message)
write_transcript_message(message, preserve_composer: true)
end
|
#write_transcript_delta(delta) ⇒ Object
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
|
# File 'lib/kward/prompt_interface.rb', line 1065
def write_transcript_delta(delta)
@mutex.synchronize do
with_synchronized_output_locked do
prepare_transcript_output_locked unless @restoring_transcript
write_transcript_text_locked(delta.to_s)
restore_composer_cursor_locked unless @restoring_transcript
end
flush_output_locked unless @restoring_transcript
end
end
|
#yes?(message, default: false) ⇒ Boolean
586
587
588
589
590
591
592
593
594
|
# File 'lib/kward/prompt_interface.rb', line 586
def yes?(message, default: false)
answer = ask("#{message} #{default ? "[Y/n]" : "[y/N]"}")
return default if answer.nil?
answer = answer.strip.downcase
return default if answer.empty?
answer.start_with?("y")
end
|